package net.geant.nmaas.orchestration.api;

import net.geant.nmaas.orchestration.DefaultAppDeploymentRepositoryManager;
import net.geant.nmaas.orchestration.exceptions.InvalidDeploymentIdException;
import net.geant.nmaas.portal.api.BaseControllerTestSetup;
import net.geant.nmaas.portal.persistent.entity.Role;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.bean.override.mockito.MockitoBean;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@ExtendWith(SpringExtension.class)
@SpringBootTest
public class OrchestratorControllerSecTest extends BaseControllerTestSetup {

    @MockitoBean
    private DefaultAppDeploymentRepositoryManager repository;

    @BeforeEach
    public void setup() {
        createMVC();
    }

    @Test
    public void shouldAuthorizeAdminProperUser() {
        String token = getValidUserTokenFor(Role.ROLE_SYSTEM_ADMIN);
        assertDoesNotThrow(() -> {
            mvc.perform(get("/api/orchestration/deployments")
                    .header("Authorization", "Bearer " + token))
                    .andExpect(status().isOk());
            when(repository.loadState(any())).thenThrow(new InvalidDeploymentIdException(""));
            mvc.perform(get("/api/orchestration/deployments/{deploymentId}/state", "id")
                    .header("Authorization", "Bearer " + token))
                    .andExpect(status().isNotFound());
        });
    }

    @Test
    public void shouldRejectNonAdminProperUser() {
        String token = getValidUserTokenFor(Role.ROLE_USER);
        assertDoesNotThrow(() -> {
            mvc.perform(get("/api/orchestration/deployments")
                    .header("Authorization", "Bearer " + token))
                    .andExpect(status().isUnauthorized());
            mvc.perform(get("/api/orchestration/deployments/{deploymentId}/state", "id")
                    .header("Authorization", "Bearer " + token))
                    .andExpect(status().isUnauthorized());
        });
    }
}
