package net.geant.nmaas.nmservice.deployment.containerorchestrators.kubernetes.entities;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import net.geant.nmaas.nmservice.deployment.containerorchestrators.kubernetes.components.helm.HelmChartIngressVariable;
import net.geant.nmaas.orchestration.entities.AppAccessMethod;
import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;

import jakarta.persistence.ElementCollection;
import jakarta.persistence.Entity;
import jakarta.persistence.EnumType;
import jakarta.persistence.Enumerated;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import java.util.HashMap;
import java.util.Map;

import static net.geant.nmaas.orchestration.entities.AppAccessMethod.ConditionType.DEPLOYMENT_PARAMETER;

/**
 * This class represents single access method to NMAAS service
 */
@Getter
@Setter
@AllArgsConstructor
@NoArgsConstructor
@Entity
@EqualsAndHashCode
@Builder
public class ServiceAccessMethod {

    public static final String DEFAULT_INTERNAL_SSH_ACCESS_USERNAME = "netops";

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @Enumerated(EnumType.STRING)
    private ServiceAccessMethodType type;

    private String name;

    private String url;

    private String protocol;

    private String condition;

    private boolean enabled;

    @ElementCollection
    @Fetch(FetchMode.SELECT)
    private Map<HelmChartIngressVariable, String> deployParameters;

    public ServiceAccessMethod(ServiceAccessMethodType type, String name, String url, String protocol, Map<HelmChartIngressVariable, String> deployParameters) {
        this.type = type;
        this.name = name;
        this.url = url;
        this.protocol = protocol;
        this.enabled = true;
        this.deployParameters = deployParameters;
    }

    public boolean isOfType(ServiceAccessMethodType type) {
        return this.getType().equals(type);
    }

    public static ServiceAccessMethod fromAppAccessMethod(AppAccessMethod appAccessMethod) {
        ServiceAccessMethod serviceAccessMethod = new ServiceAccessMethod();
        serviceAccessMethod.setType(appAccessMethod.getType());
        serviceAccessMethod.setName(appAccessMethod.getTag());
        serviceAccessMethod.setProtocol(appAccessMethod.getName());
        serviceAccessMethod.setCondition(
                DEPLOYMENT_PARAMETER.equals(appAccessMethod.getConditionType()) ? appAccessMethod.getCondition() : null
        );
        serviceAccessMethod.setEnabled(true);
        serviceAccessMethod.setUrl(null);
        serviceAccessMethod.deployParameters = new HashMap<>();
        if (appAccessMethod.getDeployParameters() != null) {
            appAccessMethod.getDeployParameters().forEach((key, value) -> {
                serviceAccessMethod.getDeployParameters().put(HelmChartIngressVariable.valueOf(key), value);
            });
        }
        return serviceAccessMethod;
    }

    public static ServiceAccessMethod copy(ServiceAccessMethod method) {
        ServiceAccessMethod copy = new ServiceAccessMethod();
        copy.setId(method.getId());
        copy.setName(method.getName());
        copy.setType(method.getType());
        copy.setProtocol(method.getProtocol());
        copy.setCondition(method.getCondition());
        copy.setEnabled(method.isEnabled());
        copy.setDeployParameters(method.getDeployParameters());
        copy.setUrl(method.getUrl());
        return copy;
    }

}