package net.geant.nmaas.orchestration.tasks.app;

import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.geant.nmaas.orchestration.DefaultAppDeploymentRepositoryManager;
import net.geant.nmaas.orchestration.Identifier;
import net.geant.nmaas.orchestration.entities.AppDeployment;
import net.geant.nmaas.orchestration.events.app.AppApplyConfigurationActionEvent;
import net.geant.nmaas.orchestration.events.app.AppVerifyConfigurationActionEvent;
import net.geant.nmaas.utils.logging.LogLevel;
import net.geant.nmaas.utils.logging.Loggable;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;

@Component
@Slf4j
@AllArgsConstructor
public class AppConfigurationVerificationTask {

    private DefaultAppDeploymentRepositoryManager appDeploymentRepositoryManager;

    private ApplicationEventPublisher eventPublisher;

    @EventListener
    @Loggable(LogLevel.INFO)
    public void trigger(AppVerifyConfigurationActionEvent event) {
        try {
            final Identifier deploymentId = event.getRelatedTo();
            final AppDeployment appDeployment = appDeploymentRepositoryManager.load(deploymentId);
            if (appDeployment.getConfiguration() != null) {
                eventPublisher.publishEvent(new AppApplyConfigurationActionEvent(this, deploymentId));
            }
        } catch (Exception e) {
            log.error("Error reported at {}", LocalDateTime.now(), e);
        }
    }

}
