package net.geant.nmaas.portal.api.security;

import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import net.geant.nmaas.portal.api.security.exceptions.TokenAuthenticationException;
import net.geant.nmaas.portal.service.TokenAuthenticationService;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;
import org.springframework.security.web.util.matcher.RequestMatcher;

import java.io.IOException;

@Slf4j
public class StatelessAuthenticationFilter extends AbstractAuthenticationProcessingFilter {

	TokenAuthenticationService tokenService;

	public StatelessAuthenticationFilter(RequestMatcher skipPaths, TokenAuthenticationService tokenService) {
		super(skipPaths);
		this.tokenService = tokenService;
	}

	@Override
	public Authentication attemptAuthentication(HttpServletRequest request, HttpServletResponse response) {
		String reqText = request.getRequestURI() != null ? request.getRequestURI() : "empty";
        log.trace("Request: {}", reqText);
		try {
			return tokenService.getAuthentication(request);
		} catch(Exception ex) {
			throw new TokenAuthenticationException("Token is not valid"  + (request.getRequestURL() != null ? request.getRequestURL() : " empty request."));
		}
	}

	@Override
	protected void successfulAuthentication(HttpServletRequest request, HttpServletResponse response, FilterChain chain,
			Authentication authResult) throws IOException, ServletException {
        log.trace("Authentication: {}", authResult);
		SecurityContext context = SecurityContextHolder.createEmptyContext();
		context.setAuthentication(authResult);
		SecurityContextHolder.setContext(context);
		chain.doFilter(request, response);
		SecurityContextHolder.clearContext();
	}

	@Override
	protected void unsuccessfulAuthentication(HttpServletRequest request, HttpServletResponse response,
			AuthenticationException failed) throws IOException, ServletException {
		log.trace("Authentication unsuccessful");
		SecurityContextHolder.clearContext();
		getFailureHandler().onAuthenticationFailure(request, response, failed);
	}
	
}