package net.geant.nmaas.portal.persistent.entity;


import lombok.AccessLevel;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

import jakarta.persistence.Column;
import jakarta.persistence.ElementCollection;
import jakarta.persistence.Entity;
import jakarta.persistence.FetchType;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.JoinTable;
import jakarta.persistence.ManyToMany;
import jakarta.persistence.Table;
import jakarta.persistence.UniqueConstraint;
import jakarta.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Entity
@Table(name = "domain_group", uniqueConstraints = {
        @UniqueConstraint(columnNames = {"name"}), @UniqueConstraint(columnNames = {"codename"})
})
@NoArgsConstructor(access = AccessLevel.PROTECTED)
@Getter
@Setter
@EqualsAndHashCode(onlyExplicitlyIncluded = true)
public class DomainGroup implements Serializable {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    Long id;

    @EqualsAndHashCode.Include
    @NotNull
    @Column(nullable = false, unique = true)
    String name;

    @EqualsAndHashCode.Include
    @NotNull
    @Column(nullable = false, unique = true)
    private String codename;

    @ElementCollection(fetch = FetchType.LAZY)
    private List<ApplicationStatePerDomain> applicationStatePerDomain = new ArrayList<>();

    @ManyToMany(fetch = FetchType.LAZY, mappedBy = "groups")
    private List<Domain> domains = new ArrayList<>();

    @ManyToMany(fetch = FetchType.LAZY)
    @JoinTable(
            name = "domain_group_managers",
            joinColumns = @JoinColumn(name = "domain_group_id"),
            inverseJoinColumns = @JoinColumn(name = "user_id")
    )
    private List<User> managers = new ArrayList<>();

    public DomainGroup(String name, String codename) {
        super();
        this.name = name;
        this.codename = name;
    }

    public DomainGroup(Long id, String name, String codename) {
        this.id = id;
        this.name = name;
        this.codename = codename;
    }

    public void addDomain(Domain domain) {
        this.domains.add(domain);
        domain.getGroups().add(this);
    }

    public void removeDomain(Domain domain) {
        this.domains.remove(domain);
        domain.getGroups().remove(this);
    }

    public void addApplicationState(ApplicationBase applicationBase) {
        this.addApplicationState(applicationBase, true);
    }

    public void addApplicationState(ApplicationBase applicationBase, boolean enabled) {
        this.addApplicationState(new ApplicationStatePerDomain(applicationBase, enabled));
    }

    public void addApplicationState(ApplicationStatePerDomain appState) {
        if (!this.applicationStatePerDomain.stream().map(ApplicationStatePerDomain::getApplicationBase)
                .map(ApplicationBase::getId).collect(Collectors.toList()).contains(appState.getApplicationBase().getId())) {
            this.applicationStatePerDomain.add(appState);
        }
    }

}
