package net.geant.nmaas.portal.persistent.entity;

import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.ManyToOne;
import lombok.AccessLevel;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import org.hibernate.envers.Audited;

import java.io.Serializable;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.util.Base64;

@Entity
@Getter
@Setter
@NoArgsConstructor(access = AccessLevel.PROTECTED)
@EqualsAndHashCode
@Slf4j
@Audited
public class SSHKeyEntity implements Serializable {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    private String name;

    @Column(columnDefinition="TEXT")
    private String keyValue;

    // base64 encoded SHA256
    @EqualsAndHashCode.Exclude
    private String fingerprint;

    @ManyToOne
    @OnDelete(action = OnDeleteAction.CASCADE)
    @EqualsAndHashCode.Exclude
    private User owner;

    public SSHKeyEntity(User owner, String name, String keyValue) {
        this.owner = owner;
        this.name = name;
        this.keyValue = keyValue;
        this.fingerprint = "fingerprint";

        // generate fingerprint
        // https://stackoverflow.com/questions/51059782/how-to-calculate-fingerprint-from-ssh-rsa-public-key-in-java
        String temp = "";
        String[] elements = keyValue.split(" ");
        if(elements.length>1) {
            temp = elements[1];
        } else {
            temp = elements[0];
        }

        try {
            MessageDigest digest = MessageDigest.getInstance("SHA-256");
            byte[] result = digest.digest(Base64.getDecoder().decode(temp));
            this.fingerprint = Base64.getEncoder().encodeToString(result);
        } catch (NoSuchAlgorithmException e) {
            log.warn(String.format("Exception caught (%s)", e.getMessage()));
            log.warn(Arrays.toString(e.getStackTrace()));
        }
    }

}
