package net.geant.nmaas.portal.api.market;

import net.geant.nmaas.portal.api.exception.MissingElementException;
import net.geant.nmaas.portal.persistent.entity.Application;
import net.geant.nmaas.portal.persistent.entity.User;
import net.geant.nmaas.portal.service.ApplicationBaseService;
import net.geant.nmaas.portal.service.ApplicationService;
import net.geant.nmaas.portal.service.UserService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.modelmapper.ModelMapper;

import java.util.Optional;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

class AppBaseControllerTest {

    private final ModelMapper modelMapper = new ModelMapper();
    private final ApplicationService appService = mock(ApplicationService.class);
    private final UserService userService = mock(UserService.class);
    private final ApplicationBaseService appBaseService = mock(ApplicationBaseService.class);

    private AppBaseController appBaseController;

    private final Application app = new Application(1L, "defaultApp", "1.1");
    private final User user = new User("admin", true);

    @BeforeEach
    void setup() {
        this.appBaseController = new AppBaseController(modelMapper, userService, appService, appBaseService);
    }

    @Test
    void shouldGetApp() {
        when(appService.findApplication(anyLong())).thenReturn(Optional.of(app));
        Application result = this.appBaseController.getApp(1L);
        assertEquals(app.getName(), result.getName());
        assertEquals(app.getVersion(), result.getVersion());
    }

    @Test
    void shouldNotGetAppWhenNullId() {
        assertThrows(MissingElementException.class, () -> this.appBaseController.getApp(null));
    }

    @Test
    void shouldNotGetNotExistingApp() {
        when(appService.findApplication(anyLong())).thenReturn(Optional.empty());
        assertThrows(MissingElementException.class, () -> this.appBaseController.getApp(1L));
    }

    @Test
    void shouldGetUserByUsername() {
        when(userService.findByUsername("admin")).thenReturn(Optional.of(user));
        User admin = appBaseController.getUser("admin");
        assertEquals(user.getUsername(), admin.getUsername());
    }

    @Test
    void shouldNotGetUserByNullUsername() {
        assertThrows(MissingElementException.class, () -> appBaseController.getUser((String) null));
    }

    @Test
    void shouldNotGetNotExistingUserByUsername() {
        when(userService.findByUsername(anyString())).thenReturn(Optional.empty());
        assertThrows(MissingElementException.class, () -> appBaseController.getUser("user1"));
    }

    @Test
    void shouldGetUserById() {
        when(userService.findById(1L)).thenReturn(Optional.of(user));
        User admin = appBaseController.getUser(1L);
        assertEquals(user.getUsername(), admin.getUsername());
    }

    @Test
    void shouldNotGetUserByNullId() {
        assertThrows(MissingElementException.class, () -> appBaseController.getUser((Long) null));
    }

    @Test
    void shouldNotGetNotExistingUserById() {
        when(userService.findById(anyLong())).thenReturn(Optional.empty());
        assertThrows(MissingElementException.class, () -> appBaseController.getUser(1L));
    }

}
